#![allow(dead_code)]

use std::sync::Arc;
use tokio::sync::{mpsc::Sender, RwLock};
use futures::stream::SplitSink;
use hyper::upgrade::Upgraded;
use hyper_util::rt::TokioIo;
use tokio_tungstenite::{tungstenite::Message, WebSocketStream};

pub mod code; pub use code::*;
pub mod packet; pub use packet::*;

#[derive(Clone)]
pub enum QRPacketData {
    None,

    QConn {
        tx:Sender<QRPacket>,
        stream:Arc<RwLock<SplitSink<WebSocketStream<TokioIo<Upgraded>>, Message>>>,
    },
    RConn,

    QDisconn,

    QHello,
    RHello(PacketHelloResponse),

    QRegister(PacketRegister),
    RRegister(PacketRegisterResponse),

    QAuth(PacketAuth),
    RAuth(PacketAuthResponse),

    QAuthResume(PacketAuthResume),
    RAuthResume(PacketAuthResumeResponse),

    QAuthRevoke,

    QUserList,
    RUserList(PacketUserListResponse),

    QUserProfile(PacketUserProfile),
    RUserProfile(PacketUserProfileResponse),

    QAccountInfo(PacketAccountInfo),
    RAccountInfo(PacketAccountInfoResponse),
    QAccountUpdate(PacketAccountUpdate),
    RAccountUpdate(PacketAccountUpdateResponse),

    QSessionList(PacketSessionList),
    RSessionList(PacketSessionListResponse),

    QSessionView(PacketSessionView),
    RSessionView(PacketSessionViewResponse),

    QSessionResign(PacketSessionRetire),

    QSessionLeave,

    QGameState(PacketGameState),
    RGameState(PacketGameStateResponse),

    GameMessage(PacketGameMessage),

    QChallenge(PacketChallenge),
    
    QChallengeAnswer(PacketChallengeAnswer),
    RChallengeAnswer(PacketChallengeAnswerResponse),

    QChallengeList,
    RChallengeList(PacketChallengeListResponse),

    QInviteAcquire,
    RInviteAcquire(PacketInviteAcquireResponse),

    QInviteList,
    RInviteList(PacketInviteListResponse),

    TestResult(PacketTestResult),
}

#[derive(Clone)]
pub struct QRPacket {
    pub id:u32,
    pub data:QRPacketData,
}
impl QRPacket {
    pub fn new(id:u32, data:QRPacketData) -> Self
    {
        Self { id, data }
    }
}
