use crate::util::pack::{pack_u8, pack_u16};

use super::Packet;

#[derive(Clone)]
pub struct PacketUserListData {
    pub handle:String,
    pub is_online:bool,
}

#[derive(Clone)]
pub struct PacketUserListResponse {
    pub status:u16,
    pub records:Vec<PacketUserListData>,
}
impl PacketUserListResponse {
    pub fn new() -> Self
    {
        Self {
            status:0,
            records:Vec::new(),
        }
    }
}
impl Packet for PacketUserListResponse {
    type Data = Self;

    fn encode(&self) -> Vec<u8>
    {
        let mut result = pack_u16(self.status as u16);

        // Records
        result.append(&mut pack_u16(self.records.len() as u16));
        for record in &self.records {
            let mut chunk = Vec::new();

            // Flags
            let mut flags = 0u8;
            flags |= record.is_online as u8;
            chunk.append(&mut pack_u8(flags));

            // Handle
            let mut bytes = record.handle.as_bytes().to_vec();
            chunk.append(&mut pack_u8(bytes.len() as u8));
            chunk.append(&mut bytes);

            result.append(&mut chunk);
        }
        result
    }
}
